@@ -7,20 +7,19 @@
 #   Peter Krempa <pkrempa@redhat.com>
 
 import libvirt
-import sys
 from xml.dom import minidom
 
 try:
     conn = libvirt.openReadOnly(None)
 except libvirt.libvirtError:
     print('Failed to connect to the hypervisor')
-    sys.exit(1)
+    exit(1)
 
 try:
     capsXML = conn.getCapabilities()
 except libvirt.libvirtError:
     print('Failed to request capabilities')
-    sys.exit(1)
+    exit(1)
 
 caps = minidom.parseString(capsXML)
 host = caps.getElementsByTagName('host')[0]
@@ -30,13 +29,17 @@ total_cpus = cells.getElementsByTagName('cpu').length
 socketIds = []
 siblingsIds = []
 
-socketIds = [ proc.getAttribute('socket_id')
-              for proc in cells.getElementsByTagName('cpu')
-              if proc.getAttribute('socket_id') not in socketIds ]
+socketIds = [
+    proc.getAttribute('socket_id')
+    for proc in cells.getElementsByTagName('cpu')
+    if proc.getAttribute('socket_id') not in socketIds
+]
 
-siblingsIds = [ proc.getAttribute('siblings')
-                for proc in cells.getElementsByTagName('cpu')
-                if proc.getAttribute('siblings') not in siblingsIds ]
+siblingsIds = [
+    proc.getAttribute('siblings')
+    for proc in cells.getElementsByTagName('cpu')
+    if proc.getAttribute('siblings') not in siblingsIds
+]
 
 print("Host topology")
 print("NUMA nodes:", cells.getAttribute('num'))
